'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.getConfigFilePath = getConfigFilePath;
exports.findCrawlerParser = findCrawlerParser;
exports.getFolderNameFromPath = getFolderNameFromPath;

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _matmanCrawler = require('matman-crawler');

var _config = require('../config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * 从指定目录开始向上查找 config 配置文件。
 *
 * @param {String} configPath 起始路径，如果非法路径，则默认从当前文件的路径向上查找
 * @return {String}
 */
function getConfigFilePath(configPath) {
    // 非法的路径时，默认为本文件夹的 dirname
    if (!configPath || !_fs2.default.existsSync(configPath)) {
        // console.log('Unknow configPath=' + configPath);
        configPath = __dirname;
    }

    // 如果不是 config file，则从当前路径往上找 config file
    if (configPath.indexOf(_config.MATMAN_CONFIG_FILE) < 0) {
        var result = _search(configPath);

        // 如果没有找到，则直接返回空路径即可
        if (!result) {
            return '';
        }

        configPath = _path2.default.resolve(result, _config.MATMAN_CONFIG_FILE);
    }

    return configPath;
}

/**
 * 通过某个路径，找到 crawlerParser 对象
 * @param {String} basePath 起始路径
 * @return {null | CrawlerParser}
 */
function findCrawlerParser(basePath) {
    // 获得 matman.config.js 的文件路径
    var configFilePath = getConfigFilePath(basePath);

    // 如果不是 config file，则从当前路径往上找 config file
    if (!configFilePath) {
        console.error('Can not find config file by basePath=' + basePath);
        return null;
    }

    // 获取 matman.config.js 中的配置项
    var configData = require(configFilePath) || {};

    // 根据配置内容获得 crawlerParser 的对象
    var crawlerParser = new _matmanCrawler.CrawlerParser(configData.rootPath, configData);

    // 如果参数不合理
    var checkResult = crawlerParser.check();
    if (!checkResult.result) {
        console.error(checkResult.msg);
        return null;
    }

    return crawlerParser;
}

function getFolderNameFromPath(targetPath) {
    // 路径为 ./xxx 类型时要先去掉 ./，然后再将所有的 / 替换为 _
    return targetPath.replace(/^\.([^[\\||\/])*[\\||\/]/gi, '').replace(new RegExp(_path2.default.sep.replace(/\\/gi, '\\\\'), 'gi'), '_').replace(/\./, '_');
}

/**
 * 在指定的路径中，获得配置文件的路径。
 *
 * 寻找的规则：从指定的路径开始，依次往上级目录查找，一直到根目录为止，如果找到，则停止。
 *
 * @param {String} [targetPath] 目标路径
 * @return {String}
 */
function _search(targetPath) {
    var currDir = targetPath || process.cwd();
    var isExist = true;

    while (!_fs2.default.existsSync(_path2.default.join(currDir, _config.MATMAN_CONFIG_FILE))) {
        currDir = _path2.default.join(currDir, '../');

        // unix跟目录为/， win32系统根目录为 C:\\格式的
        if (currDir === '/' || /^[a-zA-Z]:\\$/.test(currDir)) {
            isExist = false;
            // console.log('未找到 ' + configFileName);
            break;
        }
    }

    return isExist ? currDir : '';
}
//# sourceMappingURL=index.js.map