'use strict';

var fs = require('fs');
var istanbul = require('istanbul');
var glob = require('glob');

function getTreeSummary(collector) {
    var summarizer = new istanbul.TreeSummarizer();

    collector.files().forEach(function (key) {
        summarizer.addFileCoverageSummary(key, istanbul.utils.summarizeFileCoverage(collector.fileCoverageFor(key)));
    });

    return summarizer.getTreeSummary();
}

function getPathMap(treeSummary) {
    var ret = {};

    function walker(node) {
        ret[node.fullPath()] = node;
        node.children.forEach(function (child) {
            walker(child);
        });
    }

    walker(treeSummary.root);

    return ret;
}

/**
 * 合并不同的覆盖率文件，并生成新的结果
 *
 * @param {Object} obj 要追加的覆盖率信息
 * @param {Object} coverage 历史累积的覆盖率信息
 * @return {Object} 新累积的覆盖率信息
 */
function mergeClientCoverage(obj) {
    var coverage = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    if (!obj) {
        return;
    }

    Object.keys(obj).forEach(function (filePath) {
        var original = coverage[filePath];
        var existResult = obj[filePath];

        var result = void 0;

        if (original) {
            result = istanbul.utils.mergeFileCoverage(original, existResult);
        } else {
            result = existResult;
        }

        coverage[filePath] = result;
    });

    return coverage;
}

function getCoverageInfo(collector) {
    var treeSummary = getTreeSummary(collector);
    var pathMap = getPathMap(treeSummary);

    var filePath = treeSummary.root.fullPath();
    var outputNode = pathMap[filePath];

    return {
        lines: outputNode.metrics.lines.pct,
        statements: outputNode.metrics.statements.pct,
        functions: outputNode.metrics.functions.pct,
        branches: outputNode.metrics.branches.pct
    };
}

/**
 * 生成覆盖率结果
 *
 * @param {String} globPattern glob 这个组件使用到的文件匹配规则
 * @param {Object} [opts]
 * @param {String} [opts.dir] 覆盖率文件输出目录，默认值为 ./coverage
 */
function createE2ECoverage(globPattern) {
    var opts = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

    return new Promise(function (resolve, reject) {
        var collector = new istanbul.Collector();
        var coverage = void 0;

        // options is optional
        glob(globPattern, {}, function (err, files) {
            // console.log(files);
            if (err) {
                return reject(err);
            }

            // 如果没有过滤出文件，则返回错误
            if (!files || !files.length) {
                return reject(new Error('Not exist files by pattern=' + globPattern));
            }

            // 合并不同的覆盖率生产物
            files.forEach(function (file) {
                var fileData = JSON.parse(fs.readFileSync(file));
                coverage = mergeClientCoverage(fileData, coverage);
            });

            // 追加
            collector.add(coverage);

            // 获取关键信息
            var coverageInfo = getCoverageInfo(collector);
            // console.log(coverageInfo);

            // 输出产物
            var sync = true;
            var reporter = new istanbul.Reporter();
            reporter.dir = opts.dir || './coverage';
            reporter.add('lcovonly');
            reporter.addAll(['clover', 'cobertura']);
            reporter.write(collector, sync, function () {
                // console.log('done');
                resolve({
                    data: coverageInfo,
                    reporterDir: reporter.dir
                });
            });
        });
    });
}

module.exports = {
    createE2ECoverage: createE2ECoverage
};
//# sourceMappingURL=istanbul.js.map